@@ -7,11 +7,12 @@ package ExtUtils::Depends;
 use strict;
 use warnings;
 use Carp;
+use Config;
 use File::Find;
 use File::Spec;
 use Data::Dumper;
 
-our $VERSION = '0.308';
+our $VERSION = '0.309';
 
 sub import {
 	my $class = shift;
@@ -301,7 +302,7 @@ sub build_dll_lib {
 	my ($self, $vars) = @_;
 	$vars->{macro} ||= {};
 	$vars->{macro}{'INST_DYNAMIC_LIB'} =
-		'$(INST_ARCHAUTODIR)/$(BASEEXT)$(LIB_EXT)';
+		'$(INST_ARCHAUTODIR)/$(DLBASE)$(LIB_EXT)';
 }
 
 # Search for extra library files to link against on Windows (either native
@@ -313,6 +314,7 @@ sub find_extra_libs {
 	my %mappers = (
 		MSWin32 => sub { $_[0] . '\.(?:lib|a)' },
 		cygwin	=> sub { $_[0] . '\.dll'},
+		android => sub { $_[0] . '\.' . $Config{dlext} },
 	);
 	my $mapper = $mappers{$^O};
 	return () unless defined $mapper;
@@ -320,6 +322,10 @@ sub find_extra_libs {
 	my @found_libs = ();
 	foreach my $name (keys %{ $self->{deps} }) {
 		(my $stem = $name) =~ s/^.*:://;
+		if ( defined &DynaLoader::mod2fname ) {
+			 my @parts = split /::/, $name;
+			 $stem = DynaLoader::mod2fname([@parts]);
+		}
 		my $lib = $mapper->($stem);
 		my $pattern = qr/$lib$/;
 
@@ -334,6 +340,18 @@ sub find_extra_libs {
 
 		if ($matching_file && -f $matching_file) {
 			push @found_libs, ('-L' . $matching_dir, '-l' . $stem);
+			# Android's linker ignores the RTLD_GLOBAL flag
+			# and loads everything as if under RTLD_LOCAL.
+			# What this means in practice is that modules need
+			# to explicitly link to their dependencies,
+			# because otherwise they won't be able to locate any
+			# functions they define.
+			# We use the -l:foo.so flag to indicate that the
+			# actual library name to look for is foo.so, not
+			# libfoo.so
+			if ( $^O eq 'android' ) {
+				$found_libs[-1] = "-l:$stem.$Config{dlext}";
+			}
 			next;
 		}
 	}
@@ -356,7 +374,7 @@ sub static_lib {
 	return <<"__EOM__"
 # This isn't actually a static lib, it just has the same name on Win32.
 \$(INST_DYNAMIC_LIB): \$(INST_DYNAMIC)
-	$DLLTOOL --def \$(EXPORT_LIST) --output-lib \$\@ --dllname \$(BASEEXT).\$(SO) \$(INST_DYNAMIC)
+	$DLLTOOL --def \$(EXPORT_LIST) --output-lib \$\@ --dllname \$(DLBASE).\$(DLEXT) \$(INST_DYNAMIC)
 
 dynamic:: \$(INST_DYNAMIC_LIB)
 __EOM__
